
#include "libvdev/sglib.h"
#include "libvdev/util.h"
#include "libvdev/misc.h"
#include "libvdev/sbuf.h"

#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

const char *progname = "vdev_modprobe_wait";

typedef char* cstr;

// prototypes
SGLIB_DEFINE_VECTOR_PROTOTYPES (cstr);
SGLIB_DEFINE_VECTOR_FUNCTIONS (cstr);

int main(int argc, char *argv[])
{
        int rc;
        int fd;
        struct flock fl;
        char lockfile[1024];
        struct sglib_cstr_vector bus_drivers;
        struct sglib_cstr_vector ath9k_drivers;
        DIR *dirp;
        struct dirent *E;
        const char *busdir = "/sys/bus/usb/drivers/usb";
        const char *ath9kdir = "/sys/bus/usb/drivers/ath9k_htc";
  
        if (argc != 2) {
                fprintf(stderr, "usage: vdev_modprobe_wait <filename>\n");
                return 1;
        }

        strncpy_t (lockfile, sizeof(lockfile), argv[1], strlen(argv[1]));
   
        if (access (lockfile, F_OK ) != 0)
                return 0;
   
        fd = open(lockfile, O_RDWR);   
        memset(&fl, 0, sizeof(fl));
   
        fl.l_type = F_WRLCK; // Set a write lock
   
        fl.l_whence = SEEK_SET; // Start at beginning of file
        fl.l_start = 0; // Offset from beginning of file
        fl.l_len = 0; // Lock entire file
   
        // F_SETLK(W) ignores it; F_OFD_SETLK(W) requires it to be zero
        fl.l_pid = 0; //getpid(); // Set process ID
   
        // F_SETLKW specifies blocking mode
        rc = fcntl(fd, F_SETLKW, &fl); // Set the lock
   
        fl.l_type = F_UNLCK; // Release the lock
        rc = fcntl(fd, F_SETLK, &fl); // Unlock the file

        close(fd);
   
        unlink (lockfile);
       
        send_to_background();
        
        dirp = opendir(busdir);
        if (!dirp) {
                vdev_error ("%s error opening current directory: %s\n", progname, strerror(errno));
                exit(EXIT_FAILURE);
        };
          
        sglib_cstr_vector_init (&bus_drivers);
        
        /* Invoke partition_new() for all devices associated to hotplug partitions */
        while ((errno=0, E=readdir(dirp))) {
                if (isdigit(E->d_name[0]) && strstr(E->d_name, "-") && !strstr(E->d_name, ".")) {
                        bool already_exists = false;
                        for (int i = 0; i < sglib_cstr_vector_size(&bus_drivers); i++) {
                                if (!strcmp(E->d_name, sglib_cstr_vector_at(&bus_drivers, i))) {
                                        already_exists = true;
                                        break;
                                }
                        }
                        if (!already_exists)
                                sglib_cstr_vector_push_back(&bus_drivers, vdev_strdup_or_null(E->d_name));
                }
        }
        if (errno) {
                vdev_error ("%s error reading current directory: %s\n", progname, strerror(errno));
                closedir(dirp);
                sglib_cstr_vector_free (&bus_drivers);
                exit(EXIT_FAILURE);
        }
        closedir(dirp);
        
        for (unsigned long i = 0; i < sglib_cstr_vector_size(&bus_drivers); i++) {
                FILE *pfin = NULL;
                pid_t pid;
                int wstatus;
                sbuf_t cmd;
                sbuf_init(&cmd);
                sbuf_concat(&cmd, 3, "/bin/echo ", sglib_cstr_vector_at(&bus_drivers, i), " |sudo tee /sys/bus/usb/drivers/usb/unbind >/dev/null");
                pfin = epopen(cmd.buf, &pid);
                if (pfin) { 
                        fclose(pfin);
                        waitpid(pid, &wstatus, 0);
                }
                sbuf_free(&cmd);
        }
        
        for (unsigned long i = 0; i < sglib_cstr_vector_size(&bus_drivers); i++) {
                FILE *pfin = NULL;
                pid_t pid;
                int wstatus;
                sbuf_t cmd;
                sbuf_init(&cmd);
                sbuf_concat(&cmd, 3, "/bin/echo ", sglib_cstr_vector_at(&bus_drivers, i), " |sudo tee /sys/bus/usb/drivers/usb/bind >/dev/null");
                pfin = epopen(cmd.buf, &pid);
                if (pfin) { 
                        fclose(pfin);
                        waitpid(pid, &wstatus, 0);
                }
                sbuf_free(&cmd);
        }
        
        sglib_cstr_vector_free (&bus_drivers);
        
        dirp = opendir(ath9kdir);
        if (!dirp) {
                vdev_error ("%s error opening current directory: %s\n", progname, strerror(errno));
                exit(EXIT_FAILURE);
        };
          
        sglib_cstr_vector_init (&ath9k_drivers);
        
        /* Invoke partition_new() for all devices associated to hotplug partitions */
        while ((errno=0, E=readdir(dirp))) {
                if (isdigit(E->d_name[0]) && strstr(E->d_name, "-") && !strstr(E->d_name, ".") && !strstr(E->d_name, "..")) {
                        bool already_exists = false;
                        for (int i = 0; i < sglib_cstr_vector_size(&ath9k_drivers); i++) {
                                if (!strcmp(E->d_name, sglib_cstr_vector_at(&ath9k_drivers, i))) {
                                        already_exists = true;
                                        break;
                                }
                        }
                        if (!already_exists)
                                sglib_cstr_vector_push_back(&ath9k_drivers, vdev_strdup_or_null(E->d_name));
                }
        }
        if (errno) {
                vdev_error ("%s error reading current directory: %s\n", progname, strerror(errno));
                closedir(dirp);
                sglib_cstr_vector_free (&ath9k_drivers);
                exit(EXIT_FAILURE);
        }
        closedir(dirp);
        
        for (unsigned long i = 0; i < sglib_cstr_vector_size(&ath9k_drivers); i++) {
                FILE *pfin = NULL;
                pid_t pid;
                int wstatus;
                sbuf_t cmd;
                sbuf_init(&cmd);
                sbuf_concat(&cmd, 3, "/bin/echo ", sglib_cstr_vector_at(&ath9k_drivers, i), " |sudo tee /sys/bus/usb/drivers/ath9k_htc/unbind >/dev/null");
                pfin = epopen(cmd.buf, &pid);
                if (pfin) { 
                        fclose(pfin);
                        waitpid(pid, &wstatus, 0);
                }
                sbuf_free(&cmd);
        }
        
        for (unsigned long i = 0; i < sglib_cstr_vector_size(&ath9k_drivers); i++) {
                FILE *pfin = NULL;
                pid_t pid;
                int wstatus;
                sbuf_t cmd;
                sbuf_init(&cmd);
                sbuf_concat(&cmd, 3, "/bin/echo ", sglib_cstr_vector_at(&ath9k_drivers, i), " |sudo tee /sys/bus/usb/drivers/ath9k_htc/bind >/dev/null");
                pfin = epopen(cmd.buf, &pid);
                if (pfin) { 
                        fclose(pfin);
                        waitpid(pid, &wstatus, 0);
                }
                sbuf_free(&cmd);
        }
        
        sglib_cstr_vector_free (&ath9k_drivers);
 
        return 0;
}
